from typing import Dict, List
from pydantic import BaseModel, ConfigDict
import plotly.graph_objects as go
import pandas as pd
from IPython.display import SVG, display


class Case(BaseModel):
    status: str
    true_value: str
    pred_value: str
    pred_category: str
    observations: List
    shap_feature_importance: Dict
    lime_feature_importance: Dict
    similar_cases: List
    is_automl_prediction: bool
    model_name: str
    case_prediction_path: str
    case_prediction_svg: str
    observation_checklist: List
    policy_checklist: List
    final_decision: str
    unique_identifier: str
    tag: str
    created_at: str
    data: Dict
    similar_cases_data: List

    model_config = ConfigDict(protected_namespaces=())

    def explainability_shap_feature_importance(self):
        """Plots Shap Feature Importance chart"""
        fig = go.Figure()

        if len(list(self.shap_feature_importance.values())) < 1:
            return "No Shap Feature Importance for the case"

        if isinstance(list(self.shap_feature_importance.values())[0], dict):
            for col in self.shap_feature_importance.keys():
                fig.add_trace(
                    go.Bar(
                        x=list(self.shap_feature_importance[col].values()),
                        y=list(self.shap_feature_importance[col].keys()),
                        orientation="h",
                        name=col,
                    )
                )
        else:
            fig.add_trace(
                go.Bar(
                    x=list(self.shap_feature_importance.values()),
                    y=list(self.shap_feature_importance.keys()),
                    orientation="h",
                )
            )
        fig.update_layout(
            barmode="relative",
            height=800,
            width=800,
            yaxis_autorange="reversed",
            bargap=0.01,
            legend_orientation="h",
            legend_x=0.1,
            legend_y=1.1,
        )
        fig.show(config={"displaylogo": False})
    
    def explainability_lime_feature_importance(self):
        """Plots Lime Feature Importance chart"""
        fig = go.Figure()

        if len(list(self.lime_feature_importance.values())) < 1:
            return "No Lime Feature Importance for the case"

        if isinstance(list(self.lime_feature_importance.values())[0], dict):
            for col in self.lime_feature_importance.keys():
                fig.add_trace(
                    go.Bar(
                        x=list(self.lime_feature_importance[col].values()),
                        y=list(self.lime_feature_importance[col].keys()),
                        orientation="h",
                        name=col,
                    )
                )
        else:
            fig.add_trace(
                go.Bar(
                    x=list(self.lime_feature_importance.values()),
                    y=list(self.lime_feature_importance.keys()),
                    orientation="h",
                )
            )
        fig.update_layout(
            barmode="relative",
            height=800,
            width=800,
            yaxis_autorange="reversed",
            bargap=0.01,
            legend_orientation="h",
            legend_x=0.1,
            legend_y=1.1,
        )
        fig.show(config={"displaylogo": False})

    def explainability_prediction_path(self):
        """Explainability Prediction Path"""
        svg = SVG(self.case_prediction_svg)
        display(svg)

    def explainability_raw_data(self) -> pd.DataFrame:
        """Explainability Raw Data

        :return: raw data dataframe
        """
        raw_data_df = (
            pd.DataFrame([self.data])
            .transpose()
            .reset_index()
            .rename(columns={"index": "Feature", 0: "Value"})
        )
        return raw_data_df

    def explainability_observations(self) -> pd.DataFrame:
        """Explainability Observations

        :return: observations dataframe
        """
        observations_df = pd.DataFrame(self.observation_checklist)

        return observations_df

    def explainability_policies(self) -> pd.DataFrame:
        """Explainability Policies

        :return: policies dataframe
        """
        policy_df = pd.DataFrame(self.policy_checklist)

        return policy_df

    def explainability_decision(self) -> pd.DataFrame:
        """Explainability Decision

        :return: decision dataframe
        """
        data = {
            "True Value": self.true_value,
            "AutoML Prediction": self.pred_value,
            "Final Prediction": self.final_decision,
        }
        decision_df = pd.DataFrame([data])

        return decision_df

    def explainability_similar_cases(self) -> pd.DataFrame | str:
        """Similar Cases

        :return: similar cases dataframe
        """
        if not self.similar_cases_data:
            return "No similar cases found. Or add 'similar_cases' in components case_info()"

        similar_cases_df = pd.DataFrame(self.similar_cases_data)
        return similar_cases_df



from aryaxai.client.client import APIClient

def build_url(base_url, data_connector_name, project_name, organization_id):
    url = None
    if project_name:
        url = f"{base_url}?project_name={project_name}&link_service_name={data_connector_name}"
    elif organization_id:
        url = f"{base_url}?organization_id={organization_id}&link_service_name={data_connector_name}"
    return url

def build_list_data_connector_url(base_url, project_name, organization_id):
    url = None
    if project_name and organization_id:
        url = f"{base_url}?project_name={project_name}&organization_id={organization_id}"
    elif project_name:
        url = f"{base_url}?project_name={project_name}"
    elif organization_id:
        url = f"{base_url}?organization_id={organization_id}"
    return url


import os
from typing import Any
from pydantic import BaseModel
import json
from IPython.display import IFrame, display

from aryaxai.common.xai_uris import XAI_APP_URI


class Dashboard(BaseModel):
    config: dict
    query_params: str
    raw_data: dict | list

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.print_config()
        self.plot()

    def plot(self, width: int = "100%", height: int = 800):
        """plot the dashboard by remote url

        Args:
            width (int, optional): _description_. Defaults to 100%.
            height (int, optional): _description_. Defaults to 650.
        """
        uri = os.environ.get("XAI_APP_URL", XAI_APP_URI)
        url = f"{uri}/sdk/dashboard{self.query_params}"
        display(IFrame(src=f"{url}", width=width, height=height))

    def get_config(self) -> dict:
        """
        get the dashboard config
        """
        config_copy = {**self.config}
        config_copy.pop("metadata", None)
        return config_copy

    def get_raw_data(self) -> dict:
        """
        get the dashboard raw data
        """
        raw_data = {"created_at": self.config.get("created_at")}

        if self.config["type"] == "data_drift":
            data_drift_table = next(
                filter(
                    lambda data: data["metric"] == "DataDriftTable",
                    self.raw_data.get("metrics"),
                ),
                None,
            )
            if data_drift_table:
                for item in data_drift_table["result"].get("drift_by_columns"):
                    item.pop("current_small_distribution", None)
                    item.pop("reference_small_distribution", None)
                    item.pop("current_big_distribution", None)
                    item.pop("reference_big_distribution", None)
                    item.pop("current_mean", None)
                    item.pop("reference_std", None)
                raw_data.update(data_drift_table["result"])

        if self.config["type"] == "target_drift":
            column_drift_metric = next(
                filter(
                    lambda data: data["metric"] == "ColumnDriftMetric",
                    self.raw_data.get("metrics"),
                ),
                None,
            )
            if column_drift_metric:
                column_drift_metric["result"].pop("data", None)

                raw_data.update(column_drift_metric["result"])

        if self.config["type"] == "performance":
            classification_quality_metric = next(
                filter(
                    lambda data: data["metric"] == "ClassificationQualityMetric",
                    self.raw_data.get("metrics"),
                ),
                None,
            )
            if classification_quality_metric:
                for curr_ref in ["current", "reference"]:
                    classification_quality_metric["result"][curr_ref].pop(
                        "rate_plots_data", None
                    )
                    classification_quality_metric["result"][curr_ref].pop(
                        "plot_data", None
                    )
                raw_data.update(classification_quality_metric["result"])

        return raw_data

    def print_config(self):
        """
        pretty print the cdashboard config
        """
        config = {k: v for k, v in self.config.items() if v is not None}
        config.pop("metadata", None)
        print("Using config: ", end="")
        print(json.dumps(config, indent=4))

    def __print__(self) -> str:
        return f"Dashboard(config='{self.get_config()}')"

    def __str__(self) -> str:
        return self.__print__()

    def __repr__(self) -> str:
        return self.__print__()



from setuptools import setup, find_packages

with open('readme.md', 'r', encoding='utf-8') as f:
    long_description = f.read()
    
with open('requirements.txt', 'r', encoding='utf-8') as pr:
    install_requires = pr.readlines()

setup(
    name='aryaxai',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    description='Full stack ML Observability with AryaXAI',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='aryaxai, ML observability',
    license='MIT',
    url='https://xai.arya.ai/docs/introduction',
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(where='.'),
    python_requires='>=3.0',
    install_requires=install_requires,
    package_data={
        '': ['*.md', '*.txt'],
        'common/config': ['.env.*'],
    },
)
